"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.headers = void 0;
const logging_1 = require("../logging");
const errors_1 = require("../model/errors");
const headers = (apiKey) => ({
    Accept: 'application/json',
    'Content-Type': 'application/json',
    Authorization: `Bearer ${apiKey}`,
});
exports.headers = headers;
class Request {
    constructor(identify) {
        this.identify = identify;
    }
    renewIfFails(error, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const httpError = new errors_1.HTTPError(error.response.status, JSON.stringify(error.response.data));
            if (error.response.status != 401 || !this.identify.autoRenew) {
                throw httpError;
            }
            logging_1.log.info('Auto-Renew, renewing the token...');
            const token = yield this.identify.renewToken();
            return yield request(token);
        });
    }
    send(request) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const accessToken = this.identify.accessToken || (yield this.identify.getToken());
            try {
                const result = yield request(accessToken);
                return result.data;
            }
            catch (err) { // eslint-disable-line
                const title = ((_b = (_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.title) || 'No title';
                const detail = ((_d = (_c = err === null || err === void 0 ? void 0 : err.response) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.detail) || 'No detail';
                logging_1.log.error(`Request error: ${title} ${detail}`);
                return yield this.renewIfFails(err, request);
            }
        });
    }
}
exports.default = Request;
